/**
   This program uses the Java Reflection API to "interpret"
   scripts that instantiate objects and call methods on
   those instantiated objects.

   Here is a (informal) grammar for the scripts.

   Script     ::= Block ('\n' Block )*
   Block      ::= Object Method*
   Object     ::= '$'Integer | ClassName '\n' Integer ['\n' Argument ('\n' Argument)*]
   ClassName  ::= any valid Java class name
   Method     ::= MethodName '\n' Integer ['\n' Argument ('\n' Argument)*]
   MethodName ::= any valid Java name
   Argument   ::= '$'Integer | any valid Java primitive value or a String
   Integer    ::= any non-negative integer

   Symbols of the form $1 or $12 are used in the scripts to refer
   to previously instantiated objects. A symbol '$'Integer, with
   integer value n, would refer to the n'th created object.
*/
import java.io.*;
import java.util.Scanner;
import java.lang.reflect.*;

public class RunReflectionScript
{
   private static Object[] instantiatedObjects = new Object[50];
   private static int objIndex = 0;

   public static void main(String[] args)
   throws ClassNotFoundException, InstantiationException, IllegalAccessException,
          InvocationTargetException
   {
      String scriptFile = null;
      if (args.length >= 1)
      {
         scriptFile = args[0];
      }
      else
      {
         System.err.println("Usage: java RunReflectionScript <script-file-name>");
         System.exit(1);
      }

      try
      {
         // Open the script file.
         Scanner scriptInput = new Scanner( new File(scriptFile) );

         while ( scriptInput.hasNextLine() )
         {
            String oneLine = scriptInput.nextLine().trim();
            // This line is either the name of a class, or
            // a reference to a previously instantiated object.
            System.err.println("Object of type = " + oneLine );

            Object currentObject = null;


            // This part of the program should either look up a
            // reference to a previously instantiated object using
            // the symbolic "name" that was just read, or it should
            // instantiate a new object using the class name that
            // was just read.
            // When you use reflection to look up the constructor,
            // if the constructor is overloaded, just use the first
            // method that you find with the corrct number of parameters.


            // Now call methods on the current object
            while ( scriptInput.hasNextLine()
                 && ! (oneLine = scriptInput.nextLine().trim()).equals("") )
            {

               // This part of the program should invoke, on the current
               // object, the method whose name was just read by the loop.
               // When you use reflection to look up the method, if the
               // method is overloaded, just use the first method that
               // you find with the corrct number of parameters.

            }
         }
      }
      catch (FileNotFoundException e)
      {
         System.err.println("Could not open script file: " + scriptFile );
      }
   }//main()

}